#!/usr/bin/env bash

set -e

SCRIPT_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" >/dev/null 2>&1 && pwd )"

COMPRESS_THRS=$(($(nproc) / 4))
[[ $COMPRESS_THRS -eq 0 ]] && COMPRESS_THRS=1
[[ $COMPRESS_THRS -gt 8 ]] && COMPRESS_THRS=8

fail() {
    echo "$1" >&2
    exit 1
}

TYPE=$1
[[ "$TYPE" ]] || {
	if ! ls *.snar >&/dev/null; then
		TYPE=full
	else
		LAST_FULL=$(date --date=$(ls *.snar | sort | grep full | tail -n1 | sed 's/[.].*$//') +%s)
		LAST_BIG=$(date --date=$(ls *.snar | sort | grep '\(full\|weekly\)' | tail -n1 | sed 's/[.].*$//') +%s)
		NOW=$(date +%s)
		# do a full backup every 8 weeks
		if [[ $(($NOW - $LAST_FULL)) -ge $((3600 * 24 * 7 * 8)) ]]; then
			TYPE=full
		elif [[ $(($NOW - $LAST_BIG)) -ge $((3600 * 24 * 7)) ]]; then
			TYPE=weekly
		else
			TYPE=daily
		fi
	fi
	echo "Performing $TYPE backup (autodetected)" >&2
}
[[ "$TYPE" = "full" ]] || [[ "$TYPE" = "weekly" ]] || [[ "$TYPE" = "daily" ]] || \
    fail "usage: $0 [full|weekly|daily]"

TARGET=backup-$HOSTNAME@antea
LOCAL_HOME_ID=$(stat -c %d /home)
SOURCES_FILE=$SCRIPT_DIR/sources_$HOSTNAME
. $SOURCES_FILE
[[ $SOURCES ]] || fail "SOURCES expected to be set in $SOURCES_FILE"
TIMESTAMP=$(date +"%Y-%m-%d")

BACKUP_FILE=${TIMESTAMP}.${TYPE}.tar.xz
INCFILE_TGT=stamps/${TIMESTAMP}.${TYPE}.snar.xz
INCFILE=${TIMESTAMP}.${TYPE}.snar
INCFILE_TMP=${INCFILE}.tmp

INC_FILTER="."
if [[ "$TYPE" = "weekly" ]]; then
    INC_FILTER='\(full\|weekly\)'
fi

mkdir lock || fail "ERROR: backup already in progress (lock dir exists)"
trap "rmdir lock" EXIT

[[ -f $INCFILE_TMP ]] && fail "ERROR: backup already in progress (incremental backup file exists)"
if [[ "$TYPE" != "full" ]]; then
    LAST=$(ls *.snar | sort | grep $INC_FILTER | tail -n1)
    if [[ -f "$LAST" ]]; then
        cp --reflink=auto $LAST $INCFILE_TMP
    fi
fi

ssh $TARGET "mkdir -p stamps"
tar --create --xattrs --acls --listed-incremental=$INCFILE_TMP $SOURCES | \
    xz -T$COMPRESS_THRS -3 | ssh $TARGET "cat > $BACKUP_FILE"
xz -T$COMPRESS_THRS --keep --stdout $INCFILE_TMP | ssh $TARGET "cat > $INCFILE_TGT"
mv $INCFILE_TMP $INCFILE
